*#cpu 6812
*
* DDS MICRO-C 68HC12 Runtime Library - Prefix file
*
* Copyright 1991-2005 Dave Dunfield
* All rights reserved.
*
?REGS	EQU	$0000		Default register location for EVB12
*
	ORG	$4000		Place code in memory here
?begin	LDS	#$7FFF		Initial stack goes here
	CLR	?heap		Zero out the heap
	JSR	main		Call user module
* On exit, hang. (Change this to code suitable for your system)
exit	BRA	*
* 16 bit Multiply: D = Y * D
?mul	PSHY			Save parm1
	PSHD			Save parm2
	LDAA	1,SP		Get low of parm 1
	LDAB	3,SP		And low of parm 2
	MUL			Multiply to get result
	PSHD			Save result
	LDAA	2,SP		Get high of parm 1
	LDAB	5,SP		And low of parm 2
	MUL			Multiply to get result
	ADDB	,SP		Add to high of previous result
	STAB	,SP		And resave
	LDAA	3,SP		Get low of parm 1
	LDAB	4,SP		And high of parm 2
	MUL			Multiply to get result
	ADDB	,SP		Add to result
	STAB	,SP		And resave
	PULD			Get result from stack
	LEAS	4,SP		Drop saved parameters
	RTS
* 16 bit signed division: D = Y / D
?sdiv	PSHY			Save parm1
	PSHD			Save parm2
	EORA	2,SP		Get answer sign
	TFR	D,Y		Save
	LDD	,SP		Get parm1
	BPL	?sdiv1		Not negative
	BSR	?sdiv3		Convert
	STD	,SP		Resave
?sdiv1	LDD	2,SP		Get parm2
	BPL	?sdiv2		Not negative
	BSR	?sdiv3		Convert
	STD	2,SP		Resave
?sdiv2	TFR	Y,D		Get answer sign
	TSTA			Set flags
	PULD			Restore parm2
	PULY			Restore parm1
	BPL	?div		Non-negative answer
	BSR	?mod		Y = Y / D
	TFR	Y,D		D = Y / D
?sdiv3	COMA			Ones complment
	COMB			the 16 bit accumulator
	ADDD	#1		Two's complement (negate)
	RTS
* 16 bit unsigned Division: D = Y / D
?div	BSR	?mod		Y = Y / D
	TFR	Y,D		D = Y / D
	RTS
* 16 bit Modulus and Division: (D = Y % D, Y = Y / D)
?smod	EQU	*		Entry point for signed modulus
?mod	PSHY			Save parm1
	PSHD			Save parm2
	CLRA			Begin with zero
	CLRB			"" ""
	LDY	#17		16+1 bit shift
?mod1	ANDCC	#%11111110	No carry in
?mod2	ROL	3,SP		Parm2 * 2, CF = carry out
	ROL	2,SP		(16 bit shift)
	DEY			Reduce bit count
	BEQ	?mod3		All finished, exit
	ROLB			Shift in bit, result * 2;
	ROLA			(16 bit shift)
	CPD	,SP		Result < parm 1?
	BLO	?mod1		Yes, keep going
	SUBD	,SP		Reduce by count
	ORCC	#00000001	Set carry in
	BRA	?mod2		And proceed
?mod3	LEAS	2,SP		Drop saved parameter
	PULY			Get result
	RTS
* Shift left: D = Y << D
?shl	EXG	D,Y		D = value, Y = #bits
	CPY	#0		Special case (0 bit shift)
	BEQ	nargs		Yes, early exit
?shl1	LSLD			Shift one bit
	DEY			Reduce count
	BNE	?shl1		Not finished, continue
nargs	RTS
* Shift right: D = Y >> D
?shr	EXG	D,Y		D = value, Y = #bits
	CPY	#0		Special case (0 bit shift)
	BEQ	nargs		Yes, early exit
?shr1	LSRD			Shift one bit
	DEY			Reduce count
	BNE	?shr1		Not finished, continue
	RTS
* Signed >, >=, <, <=
?gt	PSHD		D = Y > D
	CPY	2,SP+
	BGT	?ret1
	BRA	?ret0
?ge	PSHD			D = Y >= D
	CPY	2,SP+
	BGE	?ret1
	BRA	?ret0
?lt	PSHD			D = Y < D
	CPY	2,SP+
	BLT	?ret1
	BRA	?ret0
?le	PSHD			D = Y <= D
	CPY	2,SP+
	BLE	?ret1
	BRA	?ret0
* = amd !=
?eq	PSHD			D = Y == D
	CPY	2,SP+
	BEQ	?ret1
* FALSE result, D=0, ZF=1
?ret0	CLRA
	CLRB
	RTS
?ne	PSHD			D = Y != D
	CPY	2,SP+
	BEQ	?ret0
* TRUE result, D=1, ZF=0
?ret1	LDD	#1
	RTS
* Unsigned >, >=, <, <=
?ugt	PSHD			D = Y > D
	CPY	2,SP+
	BHI	?ret1
	BRA	?ret0
?uge	PSHD			D = Y >= D
	CPY	2,SP+
	BHS	?ret1
	BRA	?ret0
?ult	PSHD			D = Y < D
	CPY	2,SP+
	BLO	?ret1
	BRA	?ret0
?ule	PSHD			D = Y <= D
	CPY	2,SP+
	BLS	?ret1
	BRA	?ret0
* Logical negation: D = !D
?not	CPD	#0
	BNE	?ret0
	BRA	?ret1
* Switch table handler
?switch	LDY	2,X+		Get label value
	BEQ	?swit1		End of table, use default
	CPD	2,X+		Does value match?
	BNE	?switch		No, try next
	JMP	,Y		Execute this case
?swit1	LDY	,X		Get default case
	JMP	,Y		Execute
* Temporary storage area...
$DD:?temp 2
