*
* Functions to perform basic arithmetic operations on arbitrarily
* large unsigned numbers. Supports: + - * / % <<1 >>1 == != < >
*
* To work on other than 32 bits, adjust the following constants:
?LSIZE	EQU	4		4 bytes (32 bits)
$DD:Longreg	4
*
* Subtract two LONG numbers: longsub(num1, num2)
*
longsub	LDX	4,SP		Get num1
	LDY	2,SP		Get num2
?bigsub	LDAB	#?LSIZE		Get byte count
	CLRA			Clear carry in
?1	LDAA	,X		Get byte from num1
	SBCA	1,Y+		Subtract num2
	STAA	1,X+		Resave dest
	DECB			Reduce count
	BNE	?1		Do them all
	ROLB			Get carry out
	CLRA			Zero high
	RTS
*
* Add two LONG numbers: longadd(num1, num2)
*
longadd	LDX	4,SP		Get num1
	LDY	2,SP		Get num2
?bigadd	LDAB	#?LSIZE		Get byte count
	CLRA			Clear carry in
?2	LDAA	,X		Get byte from num1
	ADCA	1,Y+		Add num2
	STAA	1,X+		Resave dest
	DECB			Reduce count
	BNE	?2		Do them all
	ROLB			Get carry out
	CLRA			Zero high
	RTS
*
* Shift LONG number right: longshr(num1)
*
longshr	LDX	2,SP		Get num2
?bigshr	LDAB	#?LSIZE		Get byte count
	ABX			Adjust to end
	CLRA			No carry in
?3	ROR	1,-X		Shift one byte
	DECB			Reduce count
	BNE	?3		Do them all
	ROLB			Get carry out
	RTS
*
* Shift a LONG number left: longshl(num1)
*
longshl	LDX	2,SP		Get num2
?bigshl	CLRA			No carry in
?bigslc	LDAB	#?LSIZE		Get byte count
?4	ROL	1,X+		Shift one byte
	DECB			Reduce count
	BNE	?4		Do them all
	ROLB			Get carry out
	RTS
*
* Divide two LONG numbers: longdiv(num1, num2)
*
longdiv	BSR	?10		Zero temp location
	LDAA	#(?LSIZE*8)+1	Get bit count
	STAA	?temp		Save for later
?5	CLRA			No carry in
?6	LDX	4,SP		Point to num1
	BSR	?bigslc		Shift it
	DEC	?temp		Reduce count
	BEQ	?14		End, exit
	LDX	#Longreg	Point to temp
	RORB			Set carry out
	BSR	?bigslc		Do shift
	LDY	2,SP		Point to num2
	LDX	#Longreg	Point to temp
	BSR	?bigcmp		Do compare
	TSTA			Is it less
	BNE	?5		Yes, carry zero
	LDY	2,SP		Get num2
	LDX	#Longreg	Point to temp
	BSR	?bigsub		Do subtract
	ORCC	#$01		Set carry in
	BRA	?6		And proceed
*
* Multiply two LONG numbers: longmul(num1, num2)
*
longmul	LEAS	-?LSIZE,SP	Reserve space for num3
	LEAX	,SP		Point to it
	LDY	?LSIZE+2,SP	Get num2 pointer
	BSR	?bigcpy		Copy the number
	BSR	?10		Zero temp location
?7	LDX	?LSIZE+4,SP	Point to num1
	BSR	?bigshr		Do the shift
	TSTB			Carry out?
	BEQ	?8		No, leave it
	LDX	#Longreg	Point to temp
	LEAY	,SP		Point to num3
	BSR	?bigadd		Do the addition
?8	LDX	?LSIZE+4,SP	Point to num1
	BSR	?bigtst		Test it
	BEQ	?9		End, exit
	LEAX	,SP		Point to num3
	BSR	?bigshl		Do the shift
	LEAX	,SP		Restore pointer
	BSR	?bigtst		Test it
	BNE	?7		Keep going
?9	LEAS	?LSIZE,SP	Restore stack
	LDX	4,SP		Get num1 pointer
	LDY	#Longreg	Point to temp
	BRA	?bigcpy		Copy & return
*
* Zero the temporary location
*
?10	LDX	#Longreg	Point to register
?11	LDAB	#?LSIZE		Get byte count
?12	CLR	1,X+		Zero one byte
	DECB			Reduce count
	BNE	?12		Do them all
	RTS
*
* Test a LONG number for zero: longtst(num1)
*
longtst	LDX	2,SP		Get num1
?bigtst	LDAA	#?LSIZE		Get byte count
?13	LDAB	1,X+		Get byte
	BNE	?14		Not zero - exit
	DECA			Reduce count
	BNE	?13		Exit
?14	RTS
*
* Compare two LONG numbers: longcmp(num1, num2)
*
longcmp	LDX	4,SP		Get num1
	LDY	2,SP		Get num2
?bigcmp	LDAB	#?LSIZE		Get byte count
	ABX			Adjust num1 ptr
	ABY			Adjust num2 ptr
?15	LDAA	1,-X		Get byte from source
	SUBA	1,-Y		Does it match?
	BLO	?16		Its lower
	BHI	?17		Its higher
	DECB			Reduce count
	BNE	?15		Test them all
	RTS
?16	LDD	#-1		Return -1
	RTS
?17	LDD	#1		Return 1
	RTS
*
* Copy one LONG number to another: longcpy(num1, num2)
*
longcpy	LDX	4,SP		Get num1
	LDY	2,SP		Get num2
?bigcpy	LDAB	#?LSIZE		Get byte count
?18	LDAA	1,Y+		Get byte from num2
	STAA	1,X+		Copy to num1
	DECB			Reduce count
	BNE	?18		Do them all
	RTS
*
* Set a long number
*
longset	LDX	4,SP		Get address
	BSR	?11		Zero it
	LDX	4,SP		Get address back
	LDD	2,SP		Get value
	STAB	,X		Write LOW
	STAA	1,X		Write HIGH
	RTS
